<?php

/**
 * Custom blocks
 *
 * @package CustomBlocks
 * @version 2.0
 * @author davidhs
 * @copyright 2013 by davidhs
 * @license Creative Commons Attribution 3.0 Unported License http://creativecommons.org/licenses/by/3.0/
 */

if (!defined('SMF'))
	die('Hacking attempt...');

// Gets list of settings of position.
function cbGetListSettings($pos, $start, $items_per_page, $sort)
{
	global $sourcedir;

	require_once ($sourcedir . '/Subs-CustomBlocks.php');

	$list = array();

	$cb_settings = cbGetSettings($pos, 'all');

	// Applies $sort.
	$sort_array = explode(' ', $sort);
	if (1  == count($sort_array))
		$sort_array[] = 'ASC';

	$flag_order = 'DESC' == strtoupper($sort_array[1]) ? SORT_DESC : SORT_ASC;
	$cb_settings = cb_array_column_sort($cb_settings, $sort_array[0], $flag_order, SORT_REGULAR);

	// Reindex.
	$cb_settings2 = $cb_settings;
	$cb_settings = array();
	foreach ($cb_settings2 as $row)
		$cb_settings[] = $row;

	// Paginate.
	$lim = min(count($cb_settings), $start + $items_per_page);
	for ($i = $start; $i < $lim; $i++)
		$list[] = $cb_settings[$i];

	return $list;
}

// Gets size of list of settings of position
function cbGetListSettingsSize($pos)
{
	global $sourcedir;

	require_once ($sourcedir . '/Subs-CustomBlocks.php');

	$cb_settings = cbGetSettings($pos, 'all');

	return count($cb_settings);
}

// Gets list of settings of header.
function cbGetListSettingsHeader($start, $items_per_page, $sort)
{
	return cbGetListSettings('header', $start, $items_per_page, $sort);
}

// Gets size of list of settings of header.
function cbGetListSettingsHeaderSize()
{
	return cbGetListSettingsSize('header');
}

// Gets list of settings of footer.
function cbGetListSettingsFooter($start, $items_per_page, $sort)
{
	return cbGetListSettings('footer', $start, $items_per_page, $sort);
}

// Gets size of list of settings of footer.
function cbGetListSettingsFooterSize()
{
	return cbGetListSettingsSize('footer');
}

// Admin area settings (edit).
function cbSettingsEdit()
{
	global $txt, $context, $sourcedir, $modSettings;

	require_once ($sourcedir . '/Subs-CustomBlocks.php');

	loadTemplate('CustomBlocks');

	// Sort out the context!
	$context['pos'] = isset($_GET['pos']) ? $_GET['pos'] : 'header';
	$context['id'] = isset($_GET['id']) ? (int) $_GET['id'] : 0;
	$context['settings_title'] = $context['id'] ? $txt['cb_edit_' . $context['pos'] . '_title'] : $txt['cb_add_' . $context['pos'] . '_title'];
	$start = isset($_GET[$context['pos'] . '_start']) ? (int) $_GET[$context['pos'] . '_start'] : 0;
	$sort = isset($_GET[$context['pos'] . '_sort']) ? $_GET[$context['pos'] . '_sort'] : 'order';
	$context['params_url'] = ';' . $context['pos'] . '_start=' . $start . ';' . $context['pos'] . '_sort=' . $sort . (isset($_GET[$context['pos'] . '_desc']) ? ';' . $context['pos'] . '_desc' : '');
	$context['admin_tabs']['tabs']['cb']['is_selected'] = true;
	$context['sub_template'] = 'cb_settings_edit';

	if ($context['id'])
		$context['params'] = cbGetSettings($context['pos'], $context['id']);

	// Setup the default values as needed.
	if (empty($context['params']))
		$context['params'] = cbGetSettings($context['pos']);

	// Others data.
	$context['params']['frame'] = htmlspecialchars($context['params']['frame'], ENT_QUOTES);
	$context['params']['content'] = htmlspecialchars($context['params']['content'], ENT_QUOTES);
	$context['params']['permissions'] = !empty($context['params']['permissions']) ? explode(',', $context['params']['permissions']) : array();

	$type_array = array('html', 'bbc', 'php');
	$context['params']['type_array'] = array();
	foreach ($type_array as $type)
		$context['params']['type_array'][$type] = $txt['cb_type_' . $type];

	$context['params']['permissions_users'] = array(
		'guest' => $txt['guest'],
		'user' => $txt['user'],
		'mod' => $txt['298'],
		'admin' => $txt['cb_administrator'],
	);
	$context['params']['permissions_actions'] = array(
		'view' => $txt['305'],
	);

	// Are we saving?
	if (isset($_POST['save']))
	{
		checkSession('get');

		// Validate inputs.
		$_POST['description'] = htmlspecialchars($_POST['description'], ENT_QUOTES);

		$order = isset($_POST['order']) ? (int) $_POST['order'] : 0;
		$permissions = !empty($_POST['permissions']) ? implode(',', $_POST['permissions']) : '';

		// Do the insertion/updates.
		$params = array_keys(cbGetSettings($context['pos']));

		if ($context['id'])
			$id2 = $context['id'];
		else
		{
			$id_new = 1;
			$ids = empty($modSettings['cb_' . $context['pos'] . '_ids']) ? array() : explode(',', $modSettings['cb_' . $context['pos'] . '_ids']);
			while (in_array($id_new, $ids))
				$id_new++;
			$id2 = $id_new;
		}

		$setArray = array();
		if (!$context['id'])
		{
			$ids[] = $id_new;
			sort($ids);
			$setArray['cb_' . $context['pos'] . '_ids'] = implode(',', $ids);
		}
		$pos_id2 = $context['pos'] . '_' . $id2;
		foreach ($params as $param)
			$setArray['cb_' . $pos_id2 . '_' . $param] = in_array($param, array('order', 'permissions')) ? $$param : $_POST[$param];
		cbUpdateSettings($setArray);
	}
	// Deleting?
	elseif (isset($_GET['delete']) && $context['id'])
	{
		checkSession('get');

		cbDeleteSettings('cb\\_' . $context['pos'] . '\\_' . $context['id'] . '\\_%');

		if (!empty($modSettings['cb_' . $context['pos'] . '_ids']))
		{
			$ids = explode(',', $modSettings['cb_' . $context['pos'] . '_ids']);
			$key = array_search($context['id'], $ids);
			if (is_numeric($key))
				unset($ids[$key]);

			$setArray = array();
			$setArray['cb_' . $context['pos'] . '_ids'] = implode(',', $ids);
			cbUpdateSettings($setArray);
		}
	}

	if (isset($_GET['delete']) || isset($_POST['save']))
	{
		checkSession('get');
		redirectexit('action=featuresettings;sa=cb' . $context['params_url']);
	}
}

// Admin area settings (show).
function cbSettingsShow()
{
	global $txt, $scripturl, $db_prefix, $context, $modSettings, $sourcedir, $settings;

	loadTemplate('CustomBlocks');

	$context['sub_template'] = 'cb_settings_show';
	$context['settings_title'] = $txt['cb'];

	$base_href = $scripturl . '?action=featuresettings;sa=cb';

	// Header & footer list.
	$pos_array = array('header', 'footer');
	foreach ($pos_array as $pos)
	{
		$function = 'cbGetListSettings' . ucfirst($pos) . 'Size';
		$get_count = $function();
		$items_per_page = 25;

		$headers = array(
			'order' => array(
				'label' => $txt['cb_order'],
				'style' => 'width: 5%; text-align: center;',
				'sort' => array(
					'default' => 'order',
					'reverse' => 'order DESC',
				),
			),
			'description' => array(
				'label' => $txt['cb_description'],
				'style' => 'text-align: left;',
				'sort' => array(
					'default' => 'description',
					'reverse' => 'description DESC',
				),
			),
			'type' => array(
				'label' => $txt['cb_type'],
				'style' => 'width: 30%; text-align: left;',
				'sort' => array(
					'default' => 'type',
					'reverse' => 'type DESC',
				),
			),
			'option_modify' => array(
				'style' => 'width: 10%;',
			),
			'option_delete' => array(
				'style' => 'width: 10%;',
			),
		);

		$start = isset($_REQUEST[$pos . '_start']) ? (int) $_REQUEST[$pos . '_start'] : 0;
		$sort = isset($_REQUEST[$pos . '_sort']) && isset($headers[$_REQUEST[$pos . '_sort']]) && isset($headers[$_REQUEST[$pos . '_sort']]['sort']) ? $_REQUEST[$pos . '_sort'] : 'order';
		$sort_order = $headers[$sort]['sort'][isset($_REQUEST[$pos . '_desc']) ? 'reverse' : 'default'];
		$params_url = ';' . $pos . '_start=' . $start . ';' . $pos . '_sort=' . $sort . (isset($_REQUEST[$pos . '_desc']) ? ';' . $pos . '_desc' : '');

		foreach ($headers as $col => $col_header)
		{
			if (isset($headers[$col]['sort']))
			{
				$headers[$col]['href'] = $base_href . ';' . $pos . '_sort=' . $col;
				if (!isset($_REQUEST[$pos . '_desc']) && $col == $sort)
					$headers[$col]['href'] .= ';' . $pos . '_desc';
				if ($col == $sort)
					$headers[$col]['sort_image'] = !isset($_REQUEST[$pos . '_desc']) ? 'down' : 'up';
				$headers[$col]['selected'] = $col == $sort;
			}
		}
		$num_columns = count($headers);

		$function = 'cbGetListSettings' . ucfirst($pos);
		$list = $function($start, $items_per_page, $sort_order);
		$rows = array();
		foreach ($list as $row)
		{
			$textKey = 'cb_type_' . $row['type'];
			$rows[] = array(
				'order' => array(
					'value' => $row['order'],
					'style' => 'text-align: center;',
				),
				'description' => array(
					'value' => nl2br($row['description']),
				),
				'type' => array(
					'value' => isset($txt[$textKey]) ? $txt[$textKey] : $textKey,
				),
				'option_modify' => array(
					'value' => '<a href="' . $scripturl . '?action=featuresettings;sa=cbedit;pos=' . $pos . ';id=' . $row['id'] . $params_url . '">' . $txt[17] . '</a>',
					'style' => 'text-align: center;',
				),
				'option_delete' => array(
					'value' => '<a href="' . $scripturl . '?action=featuresettings;sa=cbedit;pos=' . $pos . ';delete=1;id=' . $row['id'] . $params_url . ';sesc=' . $context['session_id'] . '" onclick="return confirm(\'' .$txt['cb_edit_delete_' . $pos . '_sure'] . '\');">' . $txt['smf138'] . '</a>',
					'style' => 'text-align: center;',
				),
			);
		}

		$context['cb_settings_show_' . $pos] = array(
			'title' => $txt['cb_' . $pos],
			'no_items_label' => $txt['cb_none_' . $pos],
			'items_per_page' => $items_per_page,
			// start?
			'page_index' => constructPageIndex($base_href . ';' . $pos . '_sort=' . $sort . (isset($_REQUEST[$pos . '_desc']) ? ';' . $pos . '_desc' : ''), $start, $get_count, $items_per_page),
			'num_columns' => $num_columns,
			'headers' => $headers,
			'rows' => $rows,
			'form' => array(
				'href' => $scripturl . '?action=featuresettings;sa=cbedit;pos=' . $pos . $params_url,
				'name' => 'cb' . ucfirst($pos),
				'hidden_fields' => array(
					'sc' => $context['session_id'],
				),
			),
			'additional_rows' => array(
				'after_title' => array(
					array(
						'value' => $txt['cb_' . $pos . '_desc'],
					),
				),
				'below_table_data' => array(
					array(
						'value' => '<input type="submit" name="new" value="' . $txt['cb_make_new_' . $pos] . '" />',
						'style' => 'text-align: right;',
					),
				),
			),
		);
	}
}

// Updates settings.
function cbUpdateSettings($setArray)
{
	global $db_prefix;

	$replaceArray = array();
	foreach ($setArray as $variable => $value)
		$replaceArray[] = "(SUBSTRING('$variable', 1, 255), SUBSTRING('$value', 1, 65534))";

	db_query("
		REPLACE INTO {$db_prefix}settings
			(variable, value)
		VALUES " . implode(',
			', $replaceArray), __FILE__, __LINE__);

	// Kill the cache - it needs redoing now, but we won't bother ourselves with that here.
	cache_put_data('modSettings', null, 90);
}

// Deletes settings.
function cbDeleteSettings($filter)
{
	global $db_prefix;

	db_query("
		DELETE FROM {$db_prefix}settings
		WHERE variable LIKE '$filter'", __FILE__, __LINE__);

	// Kill the cache - it needs redoing now, but we won't bother ourselves with that here.
	cache_put_data('modSettings', null, 90);
}

/**
 * array_column_sort
 *
 * function to sort an "arrow of rows" by its columns
 * exracts the columns to be sorted and then
 * uses eval to flexibly apply the standard
 * array_multisort function
 *
 * uses a temporary copy of the array whith "_" prefixed to the keys
 * this makes sure that array_multisort is working with an associative
 * array with string type keys, which in turn ensures that the keys
 * will be preserved.
 *
 * TODO: find a way of modifying the keys of $array directly, without using
 * a copy of the array.
 *
 * flexible syntax:
 * $new_array = array_column_sort($array [, 'col1' [, SORT_FLAG [, SORT_FLAG]]]...);
 *
 * original code credited to Ichier (www.ichier.de) here:
 * http://uk.php.net/manual/en/function.array-multisort.php
 *
 * prefixing array indeces with "_" idea credit to steve at mg-rover dot org, also here:
 * http://uk.php.net/manual/en/function.array-multisort.php
 *
 */
function cb_array_column_sort() {
	$args = func_get_args();
	$array = array_shift($args);

	// make a temporary copy of array for which will fix the
	// keys to be strings, so that array_multisort() doesn't
	// destroy them
	$array_mod = array();
	foreach ($array as $key => $value) {
		$array_mod['_' . $key] = $value;
	}

	$i = 0;
	$multi_sort_line = "return array_multisort(";
	foreach ($args as $arg) {
		$i++;
		if (is_string($arg)) {
			foreach ($array_mod as $row_key => $row) {
				$sort_array[$i][] = $row[$arg];
			}
		} else {
			$sort_array[$i] = $arg;
		}
		$multi_sort_line .= "\$sort_array[" . $i . "], ";
	}
	$multi_sort_line .= "\$array_mod);";

	eval($multi_sort_line);

	// now copy $array_mod back into $array, stripping off the "_"
	// that we added earlier.
	$array = array();
	foreach ($array_mod as $key => $value) {
		$array[substr($key, 1)] = $value;
	}
	return $array;
}

?>